library angular2.src.directives.ng_style;

import "package:angular2/annotations.dart" show Directive, onCheck;
import "package:angular2/core.dart" show ElementRef;
import "package:angular2/src/change_detection/pipes/pipe.dart" show Pipe;
import "package:angular2/src/change_detection/pipes/pipe_registry.dart"
    show PipeRegistry;
import "package:angular2/src/change_detection/pipes/keyvalue_changes.dart"
    show KeyValueChanges;
import "package:angular2/src/facade/lang.dart" show isPresent, print;
import "package:angular2/src/render/api.dart" show Renderer;

@Directive(
    selector: "[ng-style]",
    lifecycle: const [onCheck],
    properties: const ["rawStyle: ng-style"])
class NgStyle {
  PipeRegistry _pipeRegistry;
  ElementRef _ngEl;
  Renderer _renderer;
  Pipe _pipe;
  var _rawStyle;
  NgStyle(this._pipeRegistry, this._ngEl, this._renderer) {}
  set rawStyle(v) {
    this._rawStyle = v;
    this._pipe = this._pipeRegistry.get("keyValDiff", this._rawStyle);
  }
  onCheck() {
    var diff = this._pipe.transform(this._rawStyle, null);
    if (isPresent(diff) && isPresent(diff.wrapped)) {
      this._applyChanges(diff.wrapped);
    }
  }
  void _applyChanges(KeyValueChanges diff) {
    diff.forEachAddedItem((record) {
      this._setStyle(record.key, record.currentValue);
    });
    diff.forEachChangedItem((record) {
      this._setStyle(record.key, record.currentValue);
    });
    diff.forEachRemovedItem((record) {
      this._setStyle(record.key, null);
    });
  }
  void _setStyle(String name, String val) {
    this._renderer.setElementStyle(this._ngEl, name, val);
  }
}
