library angular2.src.router.router_outlet;

import "package:angular2/src/facade/async.dart" show Future, PromiseWrapper;
import "package:angular2/src/facade/lang.dart" show isBlank, isPresent;
import "package:angular2/src/core/annotations/decorators.dart"
    show Directive, Attribute;
import "package:angular2/core.dart"
    show DynamicComponentLoader, ComponentRef, ElementRef;
import "package:angular2/di.dart"
    show Injector, bind, Dependency, undefinedValue;
import "router.dart" as routerMod;
import "instruction.dart" show Instruction, RouteParams;

/**
 * A router outlet is a placeholder that Angular dynamically fills based on the application's route.
 *
 * ## Use
 *
 * ```
 * <router-outlet></router-outlet>
 * ```
 */
@Directive(selector: "router-outlet")
class RouterOutlet {
  ElementRef _elementRef;
  DynamicComponentLoader _loader;
  routerMod.Router _parentRouter;
  routerMod.Router _childRouter = null;
  ComponentRef _componentRef = null;
  Instruction _currentInstruction = null;
  RouterOutlet(this._elementRef, this._loader, this._parentRouter,
      @Attribute("name") String nameAttr) {
    // TODO: reintroduce with new // sibling routes

    // if (isBlank(nameAttr)) {

    //  nameAttr = 'default';

    //}
    this._parentRouter.registerOutlet(this);
  }
  /**
   * Given an instruction, update the contents of this outlet.
   */
  Future<dynamic> activate(Instruction instruction) {
    // if we're able to reuse the component, we just have to pass along the instruction to the

    // component's router

    // so it can propagate changes to its children
    if ((instruction == this._currentInstruction || instruction.reuse) &&
        isPresent(this._childRouter)) {
      return this._childRouter.commit(instruction.child);
    }
    this._currentInstruction = instruction;
    this._childRouter = this._parentRouter.childRouter(instruction.component);
    var params = new RouteParams(instruction.params());
    var bindings = Injector.resolve([
      bind(RouteParams).toValue(params),
      bind(routerMod.Router).toValue(this._childRouter)
    ]);
    return this
        .deactivate()
        .then((_) => this._loader.loadNextToLocation(
            instruction.component, this._elementRef, bindings))
        .then((componentRef) {
      this._componentRef = componentRef;
      return this._childRouter.commit(instruction.child);
    });
  }
  Future<dynamic> deactivate() {
    return (isPresent(this._childRouter)
        ? this._childRouter.deactivate()
        : PromiseWrapper.resolve(true)).then((_) {
      if (isPresent(this._componentRef)) {
        this._componentRef.dispose();
        this._componentRef = null;
      }
    });
  }
  Future<bool> canDeactivate(Instruction instruction) {
    // TODO: how to get ahold of the component instance here?
    return PromiseWrapper.resolve(true);
  }
}
